>From a643997d9023ed11cd45f4394c163776ef579071 Mon Sep 17 00:00:00 2001
From: Roiy Shpaner <roiy@cs.umanitoba.ca>
Date: Mon, 21 Apr 2014 23:56:48 -0500
Subject: [PATCH 04/10] MED Support - Logic

---
 modules/gui/qt4/input_manager.cpp | 266 ++++++++++++++++++++++++++++++++++++--
 modules/gui/qt4/input_manager.hpp |  66 +++++++++-
 src/Makefile.am                   |   1 +
 src/input/event.c                 |   9 ++
 src/input/event.h                 |   1 +
 src/input/input.c                 | 147 ++++++++++++++++++++-
 src/input/med_control.c           | 188 +++++++++++++++++++++++++++
 src/input/var.c                   |   2 +
 src/libvlc.h                      |   3 +
 src/playlist/engine.c             |   2 +
 10 files changed, 669 insertions(+), 16 deletions(-)
 create mode 100644 src/input/med_control.c

diff --git a/modules/gui/qt4/input_manager.cpp b/modules/gui/qt4/input_manager.cpp
index bce4002..b5e4c17 100644
--- a/modules/gui/qt4/input_manager.cpp
+++ b/modules/gui/qt4/input_manager.cpp
@@ -86,6 +86,10 @@ InputManager::InputManager( QObject *parent, intf_thread_t *_p_intf) :
     b_video      = false;
     timeA        = 0;
     timeB        = 0;
+    editState = disabled;
+    editEventStartTime = -1;
+    editEventEndTime = -1;
+    b_unsavedEvents = false;
     f_cache      = -1.; /* impossible initial value, different from all */
     registerAndCheckEventIds( IMEvent::PositionUpdate, IMEvent::FullscreenControlPlanHide );
     registerAndCheckEventIds( PLEvent::PLItemAppended, PLEvent::PLEmpty );
@@ -114,6 +118,7 @@ void InputManager::setInput( input_thread_t *_p_input )
         UpdateTeletext();
         UpdateNavigation();
         UpdateVout();
+        UpdateMED();
 
         p_item = input_GetItem( p_input );
         emit rateChanged( var_GetFloat( p_input, "rate" ) );
@@ -143,6 +148,9 @@ void InputManager::delInput()
     timeA                = 0;
     timeB                = 0;
     f_rate               = 0. ;
+    editState = disabled;
+    editEventStartTime = -1;
+    editEventEndTime = -1;
 
     if( p_input_vbi )
     {
@@ -157,6 +165,7 @@ void InputManager::delInput()
     emit rateChanged( var_InheritFloat( p_intf, "rate" ) );
     emit nameChanged( "" );
     emit chapterChanged( 0 );
+    emit eventsChanged( 0 );
     emit titleChanged( 0 );
     emit playingStatusChanged( END_S );
 
@@ -227,6 +236,12 @@ void InputManager::customEvent( QEvent *event )
         UpdateNavigation();
         UpdateName(); /* Display the name of the Chapter, if exists */
         break;
+    case IMEvent::MedSaveSuccess:
+        setMedSaveResult( true );
+        break;
+    case IMEvent::MedSaveFail:
+        setMedSaveResult( false );  
+        break;
     case IMEvent::ItemRateChanged:
         UpdateRate();
         break;
@@ -319,6 +334,12 @@ static int InputEvent( vlc_object_t *p_this, const char *,
         event = new IMEvent( IMEvent::ItemTitleChanged );
         break;
 
+    case INPUT_MED_SAVE_SUCCESS:
+        event = new IMEvent( IMEvent::MedSaveSuccess );
+        break;
+    case INPUT_MED_SAVE_FAIL:
+        event = new IMEvent( IMEvent::MedSaveFail );
+        break;
     case INPUT_EVENT_ES:
         event = new IMEvent( IMEvent::ItemEsChanged );
         break;
@@ -385,6 +406,7 @@ static int InputEvent( vlc_object_t *p_this, const char *,
 
     if( event )
         QApplication::postEvent( im, event );
+
     return VLC_SUCCESS;
 }
 
@@ -401,10 +423,10 @@ static int VbiEvent( vlc_object_t *, const char *,
 void InputManager::UpdatePosition()
 {
     /* Update position */
-    int i_length;
+    int64_t i_length;
     int64_t i_time;
     float f_pos;
-    i_length = var_GetTime(  p_input , "length" ) / CLOCK_FREQ;
+    i_length = var_GetTime(  p_input , "length" );
     i_time = var_GetTime(  p_input , "time");
     f_pos = var_GetFloat(  p_input , "position" );
     emit positionUpdated( f_pos, i_time, i_length );
@@ -636,6 +658,23 @@ void InputManager::UpdateCaching()
     }
 }
 
+void InputManager::UpdateMED()
+{
+    if( hasInput() )
+    {
+        bool b_med_exists;
+        input_Control( p_input, INPUT_GET_MED_STATE, &b_med_exists, NULL );
+
+        if ( b_med_exists )
+        {
+            eventsModified();  /* notify all related mechanisms of MED events */
+            emit setMedControls( true );  /* Show MED interface */
+        }
+        else
+            emit setMedControls( false );
+    }
+}
+
 void InputManager::requestArtUpdate( input_item_t *p_item )
 {
     bool b_current_item = false;
@@ -765,7 +804,9 @@ void InputManager::UpdateProgramEvent()
 void InputManager::sliderUpdate( float new_pos )
 {
     if( hasInput() )
+    {
         var_SetFloat( p_input, "position", new_pos );
+    }
     emit seekRequested( new_pos );
 }
 
@@ -811,6 +852,169 @@ void InputManager::sectionMenu()
     }
 }
 
+void InputManager::setMedEditMode( EditState editState )
+{
+    if( hasInput() )
+    {
+        emit setEditMode( editState );
+    }
+}
+
+void InputManager::switchUiEditMode( bool bIsSegEndEdit )
+{
+    if( hasInput() )
+    {
+        emit switchUiEdit( bIsSegEndEdit );
+    }
+}
+
+void InputManager::setEventTimes( int64_t startTime, int64_t endTime )
+{
+    if( hasInput() )
+    {
+        emit updateEventTimes( startTime, endTime );
+    }
+}
+
+void InputManager::eventsModified()
+{
+    if( hasInput() )
+    {
+        b_unsavedEvents = true;
+        emit eventsChanged( true );
+    }
+}
+
+int InputManager::saveEnhancementData( QString fileName )
+{
+    if( hasInput() )
+    {
+        char *file = NULL;
+        if ( !fileName.isEmpty() )
+            file = strdup( qtu( fileName ) );
+
+        int res = input_Control( p_input, INPUT_SAVE_ENHANCEMENT_FILE, file );
+        free( file );
+        return res;
+    }
+
+    return VLC_EGENERIC;
+}
+
+void InputManager::setMedSaveResult( bool result )
+{
+    emit medSaveResult( result );
+}
+
+int InputManager::getMedSegmentAt( seeksegment_t **p_event, int *piEventIndex, int64_t queryTime, int layerId )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_SEGMENT_AT, p_event, piEventIndex, queryTime, layerId );
+    }
+    return VLC_EGENERIC;
+}
+
+
+int InputManager::addMedLayer( char** ppsz_layer )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_ADD_LAYER, ppsz_layer );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::getMedSelectedLayers( int*** ppi_selected, int **pi_selected )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_SELECTED_LAYERS, ppi_selected, pi_selected );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::getMedLayerNames( char**** psz_layerNames, int **pi_layers )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_LAYER_NAMES, psz_layerNames, pi_layers );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::addMedSegment( seeksegment_t *p_event )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_ADD_SEGMENT, p_event );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::deleteMedSegmentAt( int iSegIdx )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_DELETE_SEGMENT_AT_IDX, iSegIdx );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::getAutoJump( bool *bAutoJump )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_AUTO_JUMP, bAutoJump );
+    }
+    return VLC_EGENERIC;
+}
+
+void InputManager::setAutoJump( bool bAutoJump )
+{
+    if( hasInput() )
+    {
+        var_SetBool( p_input, "autoJump", bAutoJump );    
+    }
+    var_SetBool( p_intf, "autoJump", bAutoJump );
+}
+
+int InputManager::getInputTime( int64_t *time )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_TIME, time );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::setInputTime( int64_t time )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_SET_TIME, time );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::getInputLength( int64_t *length )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_LENGTH, length );
+    }
+    return VLC_EGENERIC;
+}
+
+int InputManager::getInputState( int *iState )
+{
+    if( hasInput() )
+    {
+        return input_Control( p_input, INPUT_GET_STATE, iState );
+    }
+    return VLC_EGENERIC;
+}
+
 /*
  *  Teletext Functions
  */
@@ -935,26 +1139,56 @@ void InputManager::setAtoB()
     {
         timeB = var_GetTime( THEMIM->getInput(), "time"  );
         var_SetTime( THEMIM->getInput(), "time" , timeA );
-        CONNECT( this, positionUpdated( float, int64_t, int ),
-                 this, AtoBLoop( float, int64_t, int ) );
+        CONNECT( this, positionUpdated( float, int64_t, int64_t ),
+                 this, AtoBLoop( float, int64_t, int64_t ) );
     }
     else
     {
         timeA = 0;
         timeB = 0;
-        disconnect( this, SIGNAL( positionUpdated( float, int64_t, int ) ),
-                    this, SLOT( AtoBLoop( float, int64_t, int ) ) );
+        disconnect( this, SIGNAL( positionUpdated( float, int64_t, int64_t ) ),
+                    this, SLOT( AtoBLoop( float, int64_t, int64_t ) ) );
     }
     emit AtoBchanged( (timeA != 0 ), (timeB != 0 ) );
 }
 
 /* Function called regularly when in an AtoB loop */
-void InputManager::AtoBLoop( float, int64_t i_time, int )
+void InputManager::AtoBLoop( float, int64_t i_time, int64_t )
 {
     if( timeB && i_time >= timeB )
         var_SetTime( THEMIM->getInput(), "time" , timeA );
 }
 
+/* Set jump to previous event segment */
+void InputManager::prevSegment()
+{
+    if( hasInput() )
+    {
+        if( input_Control( p_input, INPUT_SET_CMD_JUMP_SEGMENT, PREV_SEG_JUMP ) != VLC_SUCCESS )
+            msg_Err( p_input, "Failed to set jump to previous segment" );
+    }
+}
+
+/* Set jump to next event segment */
+void InputManager::nextSegment()
+{
+    if( hasInput() )
+    {
+        if( input_Control( p_input, INPUT_SET_CMD_JUMP_SEGMENT, NEXT_SEG_JUMP ) != VLC_SUCCESS )
+            msg_Err( p_input, "Failed to set jump to next segment" );
+    }
+}
+
+void InputManager::updateMedLayers()
+{
+    emit layersChanged();
+}
+
+ intf_thread_t *InputManager::getIntf()
+ {
+     return p_intf;
+ }
+
 /**********************************************************************
  * MainInputManager implementation. Wrap an input manager and
  * take care of updating the main playlist input.
@@ -963,13 +1197,15 @@ void InputManager::AtoBLoop( float, int64_t i_time, int )
 
 MainInputManager::MainInputManager( intf_thread_t *_p_intf )
     : QObject(NULL), p_intf( _p_intf ),
-      random( VLC_OBJECT(THEPL), "random" ),
+      random( VLC_OBJECT(THEPL), "random" ), autoJump( VLC_OBJECT(THEPL), "autoJump" ),
       repeat( VLC_OBJECT(THEPL), "repeat" ), loop( VLC_OBJECT(THEPL), "loop" ),
       volume( VLC_OBJECT(THEPL), "volume" ), mute( VLC_OBJECT(THEPL), "mute" )
 {
     p_input = NULL;
     im = new InputManager( this, p_intf );
 
+    var_SetBool( THEPL, "isQtIntf", true );  //Tells the input thread to wait for the inputManager on start
+
     var_AddCallback( THEPL, "item-change", ItemChanged, im );
     var_AddCallback( THEPL, "activity", PLItemChanged, this );
     var_AddCallback( THEPL, "leaf-to-parent", LeafToParent, this );
@@ -1119,6 +1355,20 @@ void MainInputManager::toggleRandom()
     config_PutInt( p_intf, "random", var_ToggleBool( THEPL, "random" ) );
 }
 
+void MainInputManager::toggleAutoJump()
+{
+    bool bAutoJump = var_ToggleBool( THEPL, "autoJump" );
+    if( p_input )
+        var_ToggleBool( p_input, "autoJump" );
+
+    emit autoJumpChanged( bAutoJump );
+}
+
+void MainInputManager::setAutoJumpButton( bool bEnable)
+{
+    emit setAutoJumpBtn( bEnable );
+}
+
 void MainInputManager::notifyRandom(bool value)
 {
     emit randomChanged(value);
diff --git a/modules/gui/qt4/input_manager.hpp b/modules/gui/qt4/input_manager.hpp
index ff1a83d..e4b11ac 100644
--- a/modules/gui/qt4/input_manager.hpp
+++ b/modules/gui/qt4/input_manager.hpp
@@ -34,11 +34,23 @@
 #include "qt4.hpp"
 #include "util/singleton.hpp"
 #include "adapters/variables.hpp"
+#include "util/input_slider.hpp"
 
 #include <QObject>
 #include <QEvent>
 class QSignalMapper;
 
+#ifndef INT64_C
+#define INT64_C(c) (c ## LL)
+#endif
+
+enum EditState
+{
+    disabled = -1,
+    segmentStart = 1,
+    segmentEnd = 2
+};
+
 enum { NORMAL,    /* loop: 0, repeat: 0 */
        REPEAT_ONE,/* loop: 0, repeat: 1 */
        REPEAT_ALL,/* loop: 1, repeat: 0 */
@@ -69,6 +81,8 @@ public:
         RandomChanged,
         LoopOrRepeatChanged,
         EPGEvent,
+        MedSaveSuccess,
+        MedSaveFail,
     /*    SignalChanged, */
 
         FullscreenControlToggle = QEvent::User + IMEventTypeOffset + 20,
@@ -147,6 +161,11 @@ public:
     QString getName() { return oldName; }
     static const QString decodeArtURL( input_item_t *p_item );
 
+    /* MED */
+    EditState editState;
+    int64_t editEventStartTime;
+    int64_t editEventEndTime;
+
 private:
     intf_thread_t  *p_intf;
     input_thread_t *p_input;
@@ -159,6 +178,7 @@ private:
     float           f_cache;
     bool            b_video;
     mtime_t         timeA, timeB;
+    bool            b_unsavedEvents;
 
     void customEvent( QEvent * );
 
@@ -182,9 +202,11 @@ private:
     void UpdateRecord();
     void UpdateProgramEvent();
     void UpdateEPG();
+    void UpdateMED();
 
 public slots:
     void setInput( input_thread_t * ); ///< Our controlled input changed
+    intf_thread_t *getIntf();
     void sliderUpdate( float ); ///< User dragged the slider. We get new pos
     /* SpeedRate Rate Management */
     void reverse();
@@ -207,19 +229,47 @@ public slots:
     void activateTeletext( bool );     ///< Toggle buttons after click
     /* A to B Loop */
     void setAtoB();
+    /*  Segment Navigation */
+    void prevSegment();
+    void nextSegment();
+    void updateMedLayers();
+    /* Segment Editing */
+    void setMedEditMode( EditState setEditState );
+    void switchUiEditMode( bool bIsSegEndEdit );
+    void setEventTimes( int64_t startTime, int64_t endTime );
+    void eventsModified();
+    int saveEnhancementData( QString fileName );
+    void setMedSaveResult( bool result );
+    /* Med functionality */
+    int getMedSegmentAt( seeksegment_t **p_event, int *piEventIndex, int64_t queryTime, int layerId );
+    int addMedLayer( char** ppsz_layer );
+    int getMedSelectedLayers( int*** ppi_selected, int **pi_selected );
+    int getMedLayerNames( char**** psz_layerNames, int **pi_layers );
+    int addMedSegment( seeksegment_t *p_event );
+    int deleteMedSegmentAt( int iSegIdx );
+    int getAutoJump( bool *bAutoJump );
+    void setAutoJump( bool bAutoJump );
+
+    /* input access */
+    int getInputTime( int64_t *time );
+    int setInputTime( int64_t time );
+    int getInputLength( int64_t *length );
+    int getInputState( int *iState );
 
 private slots:
-    void AtoBLoop( float, int64_t, int );
+    void AtoBLoop( float, int64_t, int64_t );
 
 signals:
     /// Send new position, new time and new length
-    void positionUpdated( float , int64_t, int );
+    void positionUpdated( float , int64_t, int64_t );
     void seekRequested( float pos );
     void rateChanged( float );
     void nameChanged( const QString& );
     /// Used to signal whether we should show navigation buttons
     void titleChanged( bool );
     void chapterChanged( bool );
+    void eventsChanged( bool );
+    void setMedControls( bool );
     void inputCanSeek( bool );
     /// Statistics are updated
     void statisticsUpdated( input_item_t* );
@@ -248,6 +298,11 @@ signals:
     /// Program Event changes
     void encryptionChanged( bool );
     void epgChanged();
+    void layersChanged();
+    void setEditMode( EditState );
+    void switchUiEdit( bool );
+    void updateEventTimes( int64_t, int64_t );
+    void medSaveResult( bool );
 };
 
 class MainInputManager : public QObject, public Singleton<MainInputManager>
@@ -284,7 +339,7 @@ private:
     InputManager            *im;
     input_thread_t          *p_input;
     intf_thread_t           *p_intf;
-    QVLCBool random, repeat, loop;
+    QVLCBool random, repeat, loop, autoJump;
     QVLCFloat volume;
     QVLCBool mute;
 
@@ -293,12 +348,13 @@ public slots:
     void play();
     void pause();
     void toggleRandom();
+    void toggleAutoJump();
     void stop();
     void next();
     void prev();
     void prevOrReset();
     void activatePlayQuit( bool );
-
+    void setAutoJumpButton( bool );
     void loopRepeatLoopStatus();
 
 private slots:
@@ -318,6 +374,8 @@ signals:
     void randomChanged( bool );
     void repeatLoopChanged( int );
     void leafBecameParent( int );
+    void autoJumpChanged( bool );
+    void setAutoJumpBtn( bool );
 };
 
 #endif
diff --git a/src/Makefile.am b/src/Makefile.am
index 0c0a079..d2218f4 100644
--- a/src/Makefile.am
+++ b/src/Makefile.am
@@ -388,6 +388,7 @@ SOURCES_libvlc_common = \
 	input/stream_filter.c \
 	input/stream_memory.c \
 	input/subtitles.c \
+        input/med_control.c \
 	input/med_detect.c \
 	input/var.c \
 	video_output/chrono.h \
diff --git a/src/input/event.c b/src/input/event.c
index 453e04b..04b874c 100644
--- a/src/input/event.c
+++ b/src/input/event.c
@@ -162,6 +162,15 @@ void input_SendEventSeekpoint( input_thread_t *p_input, int i_title, int i_seekp
     Trigger( p_input, INPUT_EVENT_CHAPTER );
 }
 
+void input_SendEventMedSave( input_thread_t *p_input, bool result)
+{
+    if ( result )
+        Trigger( p_input, INPUT_MED_SAVE_SUCCESS );
+    else
+        Trigger( p_input, INPUT_MED_SAVE_FAIL );
+}
+
+
 void input_SendEventSignal( input_thread_t *p_input, double f_quality, double f_strength )
 {
     vlc_value_t val;
diff --git a/src/input/event.h b/src/input/event.h
index c8f1071..f54e29d 100644
--- a/src/input/event.h
+++ b/src/input/event.h
@@ -43,6 +43,7 @@ void input_SendEventSeekpoint( input_thread_t *p_input, int i_title, int i_seekp
 void input_SendEventSignal( input_thread_t *p_input, double f_quality, double f_strength );
 void input_SendEventState( input_thread_t *p_input, int i_state );
 void input_SendEventCache( input_thread_t *p_input, double f_level );
+void input_SendEventMedSave( input_thread_t *p_input, bool result);
 
 /* TODO rename Item* */
 void input_SendEventMeta( input_thread_t *p_input );
diff --git a/src/input/input.c b/src/input/input.c
index 0b8fae7..c1e10d1 100644
--- a/src/input/input.c
+++ b/src/input/input.c
@@ -351,6 +351,16 @@ static input_thread_t *Create( vlc_object_t *p_parent, input_item_t *p_item,
     p_input->p->input.i_title  = 0;
     p_input->p->input.title    = NULL;
     p_input->p->input.i_title_offset = p_input->p->input.i_seekpoint_offset = 0;
+    p_input->p->input.i_seeksegments = -1;
+    p_input->p->input.seeksegments = NULL;
+    p_input->p->input.e_jump_command = 0;
+    p_input->p->input.i_med_update_st_time = -1;
+    p_input->p->input.i_med_update_end_time = -1;
+    p_input->p->input.b_med_init = false;
+    p_input->p->input.i_layers = 0;
+    p_input->p->input.psz_layerNames = NULL;
+    p_input->p->input.i_selected = 0;
+    p_input->p->input.pi_selectedIndex = NULL;
     p_input->p->input.b_can_pace_control = true;
     p_input->p->input.b_can_rate_control = true;
     p_input->p->input.b_rescale_ts = true;
@@ -683,6 +693,20 @@ static void MainLoopStatistic( input_thread_t *p_input )
 }
 
 /**
+ * MainLoopMED
+ * It monitors and controls the MED logic to handle automatic jumps
+ */
+
+static void MainLoopMED( input_thread_t *p_input )
+{
+    bool bMedExists, bMedActive;
+    input_Control( p_input, INPUT_GET_MED_STATE, &bMedExists, &bMedActive );
+    
+    if ( bMedExists && bMedActive )  /* Med file exists and user selected content to autoplay */
+        med_control( p_input );
+}
+
+/**
  * MainLoop
  * The main input loop.
  */
@@ -751,6 +775,8 @@ static void MainLoop( input_thread_t *p_input, bool b_interactive )
             if( b_paused || !b_demux_polled )
                 i_deadline = __MIN( i_intf_update, i_statistic_update );
 
+            MainLoopMED( p_input );
+
             /* Handle control */
             for( ;; )
             {
@@ -776,7 +802,7 @@ static void MainLoop( input_thread_t *p_input, bool b_interactive )
                         continue;
                     break;
                 }
-
+                
 #ifndef NDEBUG
                 msg_Dbg( p_input, "control type=%d", i_type );
 #endif
@@ -788,7 +814,6 @@ static void MainLoop( input_thread_t *p_input, bool b_interactive )
                     b_force_update = true;
                 }
             }
-
             /* Update interface and statistics */
             i_current = mdate();
             if( i_intf_update < i_current || b_force_update )
@@ -1049,6 +1074,84 @@ static void LoadSubtitles( input_thread_t *p_input )
         var_Destroy( p_input, "sub-description" );
 }
 
+static int LoadEnhancementData( input_thread_t *p_input )
+{
+    input_source_t *med_source;
+
+    if ( !var_InheritBool( p_input, "MED_Enable" ) )
+    {
+        msg_Dbg( p_input, "Skipping Enhancement Data - Configured off");
+        return VLC_SUCCESS;
+    }
+    msg_Dbg( p_input, "Loading Enhancement Data");
+
+    /* Load Enhancement Data (Event Layers) */
+ 
+    char *psz_autopath = strdup( "" );
+    char **ppsz_med_files = med_Detect( p_input, psz_autopath,
+                                         p_input->p->p_item->psz_uri );
+    free( psz_autopath );
+                                                     
+    char *url = vlc_path2uri( ppsz_med_files[0], NULL );
+                                                     
+    med_source = InputSourceNew( p_input );                 
+    if( !med_source || !url
+     || InputSourceInit( p_input, med_source, url, "med", (SUB_CANFAIL) ) )
+    {
+        free( med_source );
+        free( url );
+        msg_Dbg( p_input, "Did not load Media Enhancement Data (.med)");
+        p_input->p->input.b_med_init = true;
+        return VLC_EGENERIC;
+    }
+    free( url );
+
+    InputSourceClean( med_source );
+    free( med_source );
+    msg_Dbg( p_input, "Finished loading Media Enhancement Data successfully");
+    p_input->p->input.b_med_init = true;
+    return VLC_SUCCESS;
+}
+
+static int SaveEnhancementData( input_thread_t *p_input, char *psz_filename )
+{
+    if ( !var_InheritBool( p_input, "MED_Enable" ) )
+    {
+        msg_Err( p_input, "Can't save Media Enhancement Data - Configured off");
+        return VLC_EGENERIC;
+    }
+    msg_Dbg( p_input, "Saving Enhancement Data");
+
+    med_data_t *p_med =
+        vlc_custom_create( p_input, sizeof( *p_med ), "med writer" );
+    if( p_med == NULL )
+        return VLC_ENOMEM;
+
+    p_med->p_input = p_input;
+    p_med->psz_uri = p_input->p->p_item->psz_uri;
+    p_med->psz_file = psz_filename;
+
+    if ( p_input->p->input.i_seeksegments < 0 ) /* need to initialize file */
+        p_med->b_init = true;
+    else
+        p_med->b_init = false;
+
+    module_t *p_mod = module_need( p_med, "med writer", "med writer", false );
+    if( p_mod )
+    {
+        free( psz_filename );
+        module_unneed( p_med, p_mod );
+        vlc_object_release( p_med );
+        input_SendEventMedSave( p_input, true );
+        return VLC_SUCCESS;
+    }
+
+    free( psz_filename );
+    vlc_object_release( p_med );
+    input_SendEventMedSave( p_input, false );
+    return VLC_EGENERIC;
+}
+
 static void LoadSlaves( input_thread_t *p_input )
 {
     char *psz = var_GetNonEmptyString( p_input, "input-slave" );
@@ -1229,6 +1332,7 @@ static int Init( input_thread_t * p_input )
     {
         StartTitle( p_input );
         LoadSubtitles( p_input );
+        LoadEnhancementData( p_input );  //Check for Media Enhancement Data and load it
         LoadSlaves( p_input );
         InitPrograms( p_input );
 
@@ -1951,6 +2055,20 @@ static bool Control( input_thread_t *p_input,
                 input_SubtitleFileAdd( p_input, val.psz_string, true );
             break;
 
+        case INPUT_CONTROL_LOAD_ENHANCEMENT_DATA:
+            if ( LoadEnhancementData( p_input ) == VLC_SUCCESS )
+                val.b_bool = true;
+            else 
+                val.b_bool = false;
+            break;
+
+        case INPUT_CONTROL_SAVE_ENHANCEMENT_DATA:
+            if ( SaveEnhancementData( p_input, val.psz_string ) == VLC_SUCCESS )
+                val.b_bool = true;
+            else 
+                val.b_bool = false;
+            break;
+
         case INPUT_CONTROL_ADD_SLAVE:
             if( val.psz_string )
             {
@@ -2371,7 +2489,6 @@ static int InputSourceInit( input_thread_t *p_input,
             msg_Warn( p_input, "cannot create a stream_t from access" );
             goto error;
         }
-
         /* Add stream filters */
         char *psz_stream_filter = var_GetNonEmptyString( p_input,
                                                          "stream-filter" );
@@ -2456,7 +2573,7 @@ static int InputSourceInit( input_thread_t *p_input,
 #else
     var_SetBool( p_input, "can-record", in->b_can_stream_record );
 #endif
-
+    
     /* get attachment
      * FIXME improve for b_preparsing: move it after GET_META and check psz_arturl */
     if( !p_input->b_preparsing )
@@ -2522,6 +2639,28 @@ static void InputSourceClean( input_source_t *in )
             vlc_input_title_Delete( in->title[i] );
         TAB_CLEAN( in->i_title, in->title );
     }
+
+    for( i = 0; i < in->i_seeksegments; i++ )
+    {
+        free( in->seeksegments[i]->psz_layerName );
+        free( in->seeksegments[i]->psz_text );
+        free( in->seeksegments[i] );
+    }
+    free( in->seeksegments );
+    in->i_seeksegments = -1;
+    in->e_jump_command = 0;
+    in->i_med_update_st_time = -1;
+    in->i_med_update_end_time = -1;
+
+
+    for( i = 0; i < in->i_layers; i++ )
+        free( in->psz_layerNames[i] );
+
+    free( in->psz_layerNames );
+    in->i_layers = 0;
+
+    free( in->pi_selectedIndex );
+    in->i_selected = 0;
 }
 
 /*****************************************************************************
diff --git a/src/input/med_control.c b/src/input/med_control.c
new file mode 100644
index 0000000..a2ddf04
--- /dev/null
+++ b/src/input/med_control.c
@@ -0,0 +1,188 @@
+/*****************************************************************************
+ * med_control.c : Media Enhancement Data control
+ *****************************************************************************
+ * Copyright (C) 2014 VLC authors and VideoLAN
+ * $Id$
+ *
+ * Authors: Roiy Shpaner <roiy@cs.umaniotba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+/**
+ *  \file
+ *  This file contains functions to monitor and control the input using data from MED(.med) files.
+ */
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include "input/input_internal.h"
+
+/* minimum time to jump between events in microsec, otherwise keep playing over the small gap */
+#define NOTICEABLE_TIME_DIFF 2000000
+
+/*****************************************************************************
+ * med_control: Monitor and control the playback according to med and user choices
+ *****************************************************************************/
+int med_control( input_thread_t *p_input )
+{
+    int64_t i_item_time, i_update_st_time, i_update_end_time;
+    bool bAutoJump;
+
+    segment_jump_e segment_jump_cmd;
+    input_Control(p_input, INPUT_GET_CMD_JUMP_SEGMENT, &segment_jump_cmd);
+
+    /* User command detected, highest priority action */
+    if (segment_jump_cmd != NO_JUMP)
+    {
+        input_Control(p_input, INPUT_SET_CMD_JUMP_SEGMENT, NO_JUMP);
+
+        int64_t jump_time;
+        
+        if (segment_jump_cmd == PREV_SEG_JUMP)
+        {
+            seeksegment_t *p_prev_med;
+            if ( input_Control( p_input, INPUT_GET_PREV_SELECTED_SEGMENT, &p_prev_med )
+                != VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Could not get previous segment" );
+                return VLC_EGENERIC;
+            }
+            
+            if ( p_prev_med )
+            {
+                jump_time = p_prev_med->i_time_offset;
+                i_update_st_time = jump_time;
+                i_update_end_time = jump_time + p_prev_med->i_time_duration;
+                vlc_seeksegment_Delete( p_prev_med );
+            }
+            else
+                return 1;
+        }
+        else if(segment_jump_cmd == NEXT_SEG_JUMP)
+        {
+            seeksegment_t *p_next_med;
+
+            if ( input_Control( p_input, INPUT_GET_NEXT_SELECTED_SEGMENT, &p_next_med )
+                != VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Could not get next segment" );
+                return VLC_EGENERIC;
+            }
+
+            if ( p_next_med )
+            {
+                jump_time = p_next_med->i_time_offset;
+                i_update_st_time = jump_time;
+                i_update_end_time = jump_time + p_next_med->i_time_duration;
+                vlc_seeksegment_Delete( p_next_med );
+            }
+            else    /* No segment to jump next to - do nothing */
+                return 1;
+        }
+        else
+        {
+            msg_Err( p_input, "Unrecognized jump command: %d", segment_jump_cmd);
+            return VLC_EGENERIC;
+        }
+
+        input_Control( p_input, INPUT_SET_TIME, jump_time );
+        input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &i_update_st_time, &i_update_end_time );
+
+        return 1;
+    }
+
+    input_Control( p_input, INPUT_GET_AUTO_JUMP, &bAutoJump );
+
+    if ( !bAutoJump )   //No need to do anything if autojump is disabled.
+        return 1;
+
+    input_Control( p_input, INPUT_GET_TIME, &i_item_time);
+    input_Control( p_input, INPUT_GET_MED_UPDATE_TIME, &i_update_st_time, &i_update_end_time );
+
+    if ( i_update_st_time <= i_item_time &&  i_item_time <= i_update_end_time )  /* Still in segment, don't jump yet */
+        return 1;
+
+    seeksegment_t *p_med;
+    if ( input_Control( p_input, INPUT_GET_SEGMENT_AT, &p_med, NULL, (int64_t)-1, -1)
+        != VLC_SUCCESS )
+    {
+        msg_Err( p_input, "Failed to get current segment");
+        return VLC_EGENERIC;
+    }
+
+    if ( !p_med )   /* Not currently in selected annotation segment, skip to the next one if needed */
+    {
+        seeksegment_t *p_next_med;
+        if ( input_Control( p_input, INPUT_GET_NEXT_SELECTED_SEGMENT, &p_next_med )
+            != VLC_SUCCESS )
+        {
+            msg_Err( p_input, "Failed to get next selected segment");
+            return VLC_EGENERIC;
+        }
+
+        int64_t skip_time = 0;
+
+        if ( p_next_med )
+        {
+            skip_time = p_next_med->i_time_offset;
+            i_update_st_time = skip_time;
+            i_update_end_time = skip_time + p_next_med->i_time_duration;
+
+            vlc_seeksegment_Delete( p_next_med );
+        }
+        else    // No next event, stop med updates
+        {       
+            if ( input_Control( p_input, INPUT_GET_LENGTH, &i_update_end_time)
+               != VLC_SUCCESS )
+            {
+                msg_Err( p_input, "Failed to get length");
+                return VLC_EGENERIC;
+            }
+
+            input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &i_item_time, &i_update_end_time );
+
+            return 1;
+        }
+        
+        if ( skip_time < i_item_time + NOTICEABLE_TIME_DIFF )
+        {
+            msg_Dbg( p_input, "Skipping time is too short (not larger than %d), keep playing",
+              NOTICEABLE_TIME_DIFF );
+            input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &i_item_time, &i_update_end_time );
+
+            return 1;
+        }
+
+        input_Control( p_input, INPUT_SET_TIME, skip_time );
+        input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &i_update_st_time ,&i_update_end_time );
+       
+        return 1;
+    } 
+
+    /* set next update time */
+    i_update_st_time = p_med->i_time_offset;
+    i_update_end_time = p_med->i_time_offset + p_med->i_time_duration;
+    input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &i_update_st_time, &i_update_end_time );
+
+    vlc_seeksegment_Delete( p_med );
+
+    return 1;
+}
+
+
diff --git a/src/input/var.c b/src/input/var.c
index 3e722f4..2f7fe57 100644
--- a/src/input/var.c
+++ b/src/input/var.c
@@ -220,6 +220,8 @@ void input_ControlVarInit ( input_thread_t *p_input )
     var_Create( p_input, "bit-rate", VLC_VAR_INTEGER );
     var_Create( p_input, "sample-rate", VLC_VAR_INTEGER );
 
+    var_Create( p_input, "autoJump", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+
     if( !p_input->b_preparsing )
     {
         /* Special "intf-event" variable. */
diff --git a/src/libvlc.h b/src/libvlc.h
index 0e769c6..e676bcf 100644
--- a/src/libvlc.h
+++ b/src/libvlc.h
@@ -243,4 +243,7 @@ void stats_CounterClean (counter_t * );
 void stats_ComputeInputStats(input_thread_t*, input_stats_t*);
 void stats_ReinitInputStats(input_stats_t *);
 
+/* MED format control functionality */ 
+int med_control( input_thread_t *input );
+
 #endif
diff --git a/src/playlist/engine.c b/src/playlist/engine.c
index 4890d0a..25530b7 100644
--- a/src/playlist/engine.c
+++ b/src/playlist/engine.c
@@ -457,6 +457,8 @@ static void VariablesInit( playlist_t *p_playlist )
     var_Create( p_playlist, "loop", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Create( p_playlist, "corks", VLC_VAR_INTEGER );
     var_AddCallback( p_playlist, "corks", CorksCallback, NULL );
+    var_Create( p_playlist, "autoJump", VLC_VAR_BOOL );
+    var_SetBool( p_playlist, "autoJump", true );
 
     var_Create( p_playlist, "rate", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
     var_AddCallback( p_playlist, "rate", RateCallback, NULL );
-- 
1.8.4.msysgit.0

